#include "main.hpp"
#if (U_ICU_VERSION_MAJOR_NUM >= 64)
#include <unicode/uformattedvalue.h>
#endif // (U_ICU_VERSION_MAJOR_NUM >= 64)

void init_uformattedvalue(py::module &m) {
#if (U_ICU_VERSION_MAJOR_NUM >= 64)
  py::enum_<UFieldCategory>(m, "UFieldCategory", py::arithmetic())
      .value("UFIELD_CATEGORY_UNDEFINED", UFIELD_CATEGORY_UNDEFINED)
      .value("UFIELD_CATEGORY_DATE", UFIELD_CATEGORY_DATE)
      .value("UFIELD_CATEGORY_NUMBER", UFIELD_CATEGORY_NUMBER)
      .value("UFIELD_CATEGORY_LIST", UFIELD_CATEGORY_LIST)
      .value("UFIELD_CATEGORY_RELATIVE_DATETIME", UFIELD_CATEGORY_RELATIVE_DATETIME)
      .value("UFIELD_CATEGORY_LIST_SPAN", UFIELD_CATEGORY_LIST_SPAN)
      .value("UFIELD_CATEGORY_DATE_INTERVAL_SPAN", UFIELD_CATEGORY_DATE_INTERVAL_SPAN)
#if (U_ICU_VERSION_MAJOR_NUM >= 69)
      .value("UFIELD_CATEGORY_NUMBER_RANGE_SPAN", UFIELD_CATEGORY_NUMBER_RANGE_SPAN)
#endif // (U_ICU_VERSION_MAJOR_NUM >= 69)
      .export_values();
#endif // (U_ICU_VERSION_MAJOR_NUM >= 64)
}
